@@ -8,6 +8,7 @@ use base qw( DB::Introspector::CommonRDB::Introspector );
 use constant SINGLE_TABLE_QUERY => 
 'SELECT LOWER(relname) AS %s, relowner AS %s FROM pg_class 
  WHERE LOWER(relname)=?';
+
 use constant ALL_TABLES_QUERY => 
 q(SELECT LOWER(relname) AS %s, relowner AS %s FROM pg_class 
   WHERE relowner>1 AND relkind='r');
@@ -64,6 +65,8 @@ use DB::Introspector::Base::CLOBColumn;
 
 use constant COLUMN_CLASS_MAPPING => {
     'bool' => 'DB::Introspector::Base::BooleanColumn',
+    'int2' => 'DB::Introspector::Base::IntegerColumn',
+    'int4' => 'DB::Introspector::Base::IntegerColumn',
     'int8' => 'DB::Introspector::Base::IntegerColumn',
     'bpchar' => 'DB::Introspector::Base::CharColumn',
     'varchar' => 'DB::Introspector::Base::StringColumn',
@@ -83,18 +86,29 @@ use constant COLUMN_QUERY =>
     ORDER BY a.attnum';
 
 use constant FOREIGN_KEYS_QUERY => 
-'SELECT tr.tgname AS NAME, tr.tgnargs AS LENGTH, tr.tgargs AS ARGUMENTS 
+'SELECT tr.tgnargs AS LENGTH, tr.tgargs AS ARGUMENTS 
  FROM pg_trigger tr, pg_type t 
  WHERE tr.tgisconstraint 
     AND tr.tgenabled 
     AND tr.tgrelid=t.typrelid 
-    AND LOWER(t.typname)=?';
+    AND tr.tgargs like ? 
+    GROUP BY tr.tgnargs, tr.tgargs';
 
 use constant PRIMARY_KEYS_QUERY =>
 'SELECT indkey AS IDS FROM  pg_index i, 
                             pg_type c 
  WHERE i.indisprimary and i.indrelid=c.typrelid and LOWER(c.typname)=?'; 
 
+use constant INDEX_LOOKUP_QUERY =>
+q(select i.relname AS name,        
+         ind.indkey AS column_ids,
+         ind.indisunique AS unique 
+  FROM pg_class t, 
+       pg_class i, 
+       pg_index ind 
+  WHERE ind.indexrelid=i.oid   
+    AND ind.indrelid=t.oid
+    AND t.relname=?);
 
 
 use constant COLUMN_ARG_DELIM => '\0';
@@ -123,23 +137,51 @@ sub get_column_lookup_statement {
     return $sth;
 }
 
+sub get_indexes_lookup_statement {
+    my $self = shift;
+
+    my $sth = $self->_introspector->dbh->prepare_cached(INDEX_LOOKUP_QUERY);
+
+    $sth->execute(lc($self->name));
+
+    return $sth;
+}
+
 sub get_foreign_keys_lookup_statement {
     my $self = shift;
 
     my $sth = $self->_introspector->dbh->prepare_cached(FOREIGN_KEYS_QUERY);
 
-    $sth->execute($self->name);
+    $sth->execute('%\000'.$self->name.'\000%');
 
     return $sth;
 }
 
+sub _lookup_dependencies {
+    my $self = shift;
+
+    my ($foreign_keys, $dependencies) = $self->_lookup_fk_triggers;
+
+    return $dependencies;
+}
 
 sub _lookup_foreign_keys {
     my $self = shift;
+    my ($foreign_keys, $dependencies) = $self->_lookup_fk_triggers;
+
+    return $foreign_keys;
+}
+
+
+sub _lookup_fk_triggers {
+    my $self = shift;
 
     my $sth = $self->get_foreign_keys_lookup_statement();
 
     my @foreign_keys;
+    my @dependencies;
+
+    my %visited_trigger;
 
     # I think the only way for us to know if we have a foreign key (as opposed
     # to another table referencing us) is to parse the arguments of the trigger
@@ -147,18 +189,33 @@ sub _lookup_foreign_keys {
     # name.  With the new dictionary tables (like pg_constraints) in Postgres
     # 7.3, I don't think we will have to do this.
     while( my $row = $sth->fetchrow_hashref('NAME_uc') ) {
-        my @column_args = split(COLUMN_ARG_DELIM, $row->{'ARGUMENTS'});
-        if( lc($column_args[1]) ne lc($self->name) ) {
+
+        # Since triggers appear more than once, but with different ids, this
+        # hack acts as a way to make these triggers unique.
+        if( $visited_trigger{$row->{ARGUMENTS}} ) {
             next;
+        } else {
+            $visited_trigger{$row->{ARGUMENTS}} = 1;
+        }
+        
+        my @column_args = split(COLUMN_ARG_DELIM, $row->{'ARGUMENTS'});
+        if( lc($column_args[1]) eq lc($self->name) ) {
+            # dealing with foreign keys referencing other tables
+            my $foreign_key = $self->get_foreign_key_class()
+                                                        ->new($self,0,%$row);
+            push(@foreign_keys, $foreign_key);
+        } elsif( lc($column_args[2]) eq lc($self->name) ) {
+            # dealing with dependencies referencing us
+            my $t = $self->_introspector->find_table($column_args[1]); 
+            my $foreign_key = $self->get_foreign_key_class()->new($t,1,%$row);
+            push(@dependencies, $foreign_key);
         }
 
-        my $foreign_key = $self->get_foreign_key_class()->new($self, %$row);
-        push(@foreign_keys, $foreign_key);
     }
 
     $sth->finish();
 
-    return \@foreign_keys;
+    return (\@foreign_keys, \@dependencies);
 }
 
 sub get_column_instance {
@@ -166,13 +223,14 @@ sub get_column_instance {
     my $name = shift;
     my $type_name = lc(shift);
 
-    my $class = COLUMN_CLASS_MAPPING()->{$type_name}
-                            || die("class not found for type:$type_name");
+    my $class = COLUMN_CLASS_MAPPING()->{$type_name} || return;
     return $class->new($name);
 }
 
 sub get_foreign_key_class { return 'DB::Introspector::Postgres::ForeignKey'; }
 
+sub get_index_class { return 'DB::Introspector::Postgres::Index'; }
+
 
 
 package DB::Introspector::Postgres::ForeignKey;
@@ -183,7 +241,7 @@ use base qw( DB::Introspector::CommonRDB::ForeignKey );
 
 sub new {
     my $class = shift;
-    my $self = $class->SUPER::new(shift());
+    my $self = $class->SUPER::new(shift(), shift());
 
     my %args = @_;
 
@@ -238,4 +296,41 @@ sub _parse_foreign_key_data {
     $self->set_foreign_column_names(\@foreign_column_names);
 }
 
+
+package DB::Introspector::Postgres::Index;
+
+use strict;
+
+use base qw( DB::Introspector::CommonRDB::Index );
+
+use constant COLUMN_ARG_DELIM => ' ';
+
+sub new {
+    my $class = shift;
+    my $table = shift;
+    my %row = @_;
+    my $self = $class->SUPER::new($table, $row{UNIQUE});
+    $self->{_name} = $row{NAME} || "";
+    $self->set_column_names([$self->_resolve_column_names($row{COLUMN_IDS})]);
+    $self;
+}
+
+sub name {
+    my $self = shift;
+    return $self->{_name};
+}
+
+sub _resolve_column_names {
+    my $self = shift;
+    my $column_ids = shift;
+    my @ids = split(COLUMN_ARG_DELIM, $column_ids);
+    my @columns = $self->table->columns;
+    return map { $columns[$_-1]->name; } @ids;
+}
+
+sub column_names {
+    my $self = shift;
+    return @{$self->{column_names}};
+}
+
 1;
